from crunch_convert._model import RequirementLanguage as RequirementLanguage
from crunch_convert.requirements_txt._formatter import \
    format_files_from_imported as format_files_from_imported
from crunch_convert.requirements_txt._formatter import \
    format_files_from_named as format_files_from_named
from crunch_convert.requirements_txt._freeze import \
    CrunchHubVersionFinder as CrunchHubVersionFinder
from crunch_convert.requirements_txt._freeze import \
    LocalSitePackageVersionFinder as LocalSitePackageVersionFinder
from crunch_convert.requirements_txt._freeze import \
    VersionFinder as VersionFinder
from crunch_convert.requirements_txt._freeze import freeze as freeze
from crunch_convert.requirements_txt._model import \
    NamedRequirement as NamedRequirement
from crunch_convert.requirements_txt._parse import \
    parse_from_file as parse_from_file
from crunch_convert.requirements_txt._whitelist import \
    CachedWhitelist as CachedWhitelist
from crunch_convert.requirements_txt._whitelist import \
    CrunchHubWhitelist as CrunchHubWhitelist
from crunch_convert.requirements_txt._whitelist import Library as Library
from crunch_convert.requirements_txt._whitelist import \
    LocalWhitelist as LocalWhitelist
from crunch_convert.requirements_txt._whitelist import \
    MultipleLibraryAliasCandidateException as \
    MultipleLibraryAliasCandidateException
from crunch_convert.requirements_txt._whitelist import Whitelist as Whitelist



from dataclasses import dataclass, field
from typing import List

from crunch_convert._model import RequirementLanguage
from crunch_convert.notebook._utils import list_of_string_factory


@dataclass()
class NamedRequirement:
    name: str
    extras: List[str] = field(default_factory=list_of_string_factory)
    specs: List[str] = field(default_factory=list_of_string_factory)
    language: RequirementLanguage = RequirementLanguage.PYTHON



from string import whitespace
from textwrap import indent
from typing import List


def list_of_string_factory() -> List[str]:
    return []


def print_indented(text: str):
    indented = indent(text, "   | ", lambda x: True)

    if indented.endswith("\n"):
        indented = indented[:-1]

    print(indented)


def cut_crlf(input: str):
    input = input.replace("\r", "")

    if input.endswith('\n'):
        input = input[:-1]

    return input


def strip_hashes(input: str) -> str:
    return input.strip(whitespace + "#")


def format_requirement_line(name: str, extras: List[str], specs: List[str]) -> str:
    line = name

    if extras:
        line += f"[{','.join(extras)}]"

    if specs:
        line += f"{','.join(specs)}"

    return line



from typing import Optional

import libcst


def is_r_import(original_node: libcst.CSTNode) -> Optional[str]:
    is_assign = (
        isinstance(original_node, libcst.Assign)
        and len(original_node.targets) == 1
        and isinstance(original_node.targets[0].target, libcst.Name)
    )

    is_ann_assign = (
        isinstance(original_node, libcst.AnnAssign)
        and isinstance(original_node.target, libcst.Name)
    )

    if not (is_assign or is_ann_assign):
        return None

    call = original_node.value  # type: ignore
    if not isinstance(call, libcst.Call):
        return None

    func = call.func
    if not isinstance(func, libcst.Name):
        return None

    if func.value != "importr":
        return None

    if len(call.args) == 0:
        return None

    first_arg = call.args[0].value
    if not isinstance(first_arg, libcst.SimpleString):
        return None

    raw_string = first_arg.value
    if len(raw_string) <= 2:
        # only contains quotes
        return None

    return raw_string[1:-1]
